@@ -36,6 +36,11 @@ AptInterface::AptInterface(MainWindow* w_)
 	iReady = false;
 	iError = tr("Undefined error");
 
+	QDir logdir(KlogFileDir);
+	logdir.mkpath(KlogFileDir);
+	QFile logfile(KlogFileName);
+	logfile.remove();
+
 	getRepositories();
 	iNumAutoUpdates = 0;
 }
@@ -55,7 +60,7 @@ bool AptInterface::updateCatalogs()
 
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
-	runParameters << "update";
+	runParameters << "-q" << "update";
 
 	iRunProc->start(runBinary,runParameters);
 	return true;
@@ -132,7 +137,8 @@ void AptInterface::procFinished(int exitCode, QProcess::ExitStatus exitStatus)
 			iError.prepend(tr("Exit code "));
 			QString s;
 			s.setNum(exitStatus);
-			iError.append(", status " + s);
+			iError.append(", status " + s );
+			iError.append("<br>See the log for details (you can access it from the menu)");
 			iMainWindow->operationFinished(lastMode, false, iError);
 		}
 	} else {
@@ -203,41 +209,16 @@ bool AptInterface::parseAptCacheOutput()
 {
 	QList<QByteArray> lines = iRunProc->readAllStandardOutput().split('\n');
 
-	Package* pkg = 0;
+	Package* pkg = 0;	
+	multiLine multiline=MultiLineNone;
 	for( int i=0; i<lines.count(); i++)
 	{
-		if( lines.at(i).startsWith("Package: ") )
-		{
-			QString pkgname = lines.at(i).mid(9).trimmed();
-			if( pkgname==0 || pkgname == "" ) {
-				iError = tr("Empty package name");
-				//std::cout << "Encountered empty package name!" << std::endl;
-				return false;
-			}
-			pkg = new Package(pkgname);
-			iPackages.insert(pkgname,pkg);
-		}
-		if( lines.at(i).startsWith("Section: ") && pkg != 0 )
-		{
-			QString sect = lines.at(i).mid(9).trimmed();
-			pkg->setSection(sect);
-		}
-		if( lines.at(i).startsWith("Version: ") && pkg != 0 )
-		{
-			QString ver = lines.at(i).mid(9).trimmed();
-			pkg->setVersionAvailable(ver);
-		}
-		if( lines.at(i).startsWith("Maemo-Display-Name: ") && pkg != 0 )
-		{
-			QString mdn = lines.at(i).mid(20).trimmed();
-			pkg->setMaemoDisplayName(mdn);
-		}
-		if( lines.at(i).startsWith("Description: ") && pkg != 0 )
-		{
-			QString ds = lines.at(i).mid(13).trimmed();
-			pkg->setDescShort(ds);
-		}
+		QByteArray line = lines.at(i);
+		if( !processPackageDataLine(pkg, line, multiline, false) )
+			return false;
 	}
+	if( pkg!=0 ) pkg->updateStatus(); // for the last package
+
 	//std::cout << "apt-cache: read " << iPackages.count() << " available packages" << std::endl;
 	return true;
 }
@@ -252,56 +233,110 @@ bool AptInterface::readDpkgDatabase()
 	}
 
 	Package* pkg = 0;
-	int newpkg=0;
-	int totpkg=0;
+	multiLine multiline=MultiLineNone;
 
 	while (!db.atEnd()) {
 		QByteArray line = db.readLine();
-
-		if( line.startsWith("Package: ") )
-		{
-			QString pkgname = line.mid(9).trimmed();
-			if( pkgname==0 || pkgname == "" ) {
-				iError = tr("Empty package name");
-				//std::cout << "Encountered empty package name!" << std::endl;
-				return false;
-			}
-			pkg = iPackages.value(pkgname,0);
-			if( pkg==0 ) {
-				pkg = new Package(pkgname);
-				iPackages.insert(pkgname,pkg);
-				pkg->setLocal(true);
-				newpkg++;
-			}
-			totpkg++;
-		}
-		if( line.startsWith("Status: ") && pkg != 0 )
-		{
-			QString stat = line.mid(8).trimmed();
-			if( stat == "install ok installed" )
-				pkg->setInstalled(true);
-		}
-		if( line.startsWith("Section: ") && pkg != 0 && pkg->section()=="" )
-		{
-			QString sect = line.mid(9).trimmed();
-			pkg->setSection(sect);
-		}
-		if( line.startsWith("Version: ") && pkg != 0 )
-		{
-			QString ver = line.mid(9).trimmed();
-			pkg->setVersionInstalled(ver);
-		}
-		if( line.startsWith("Description: ") && pkg != 0 && pkg->descShort()=="" )
-		{
-			QString ds = line.mid(13).trimmed();
-			pkg->setDescShort(ds);
-		}
-
+		if( !processPackageDataLine(pkg, line, multiline, true) )
+			return false;
 	}
+	if( pkg!=0 ) pkg->updateStatus(); // for the last package
 
 	db.close();
 
-	//std::cout << "Read dpkg database info for " << totpkg << " packages (" << newpkg << " new)" << std::endl;
+	//std::cout << "dpkg: now " << iPackages.count() << " packages total" << std::endl;
+
+	return true;
+}
+
+bool AptInterface::processPackageDataLine(Package*& pkg, QByteArray& line, multiLine& multiline, bool dpkg)
+{
+	if( !line.startsWith(" ") )
+		line = line.trimmed();
+
+	if( line.startsWith("Package: ") )
+	{
+		if( pkg!=0 ) pkg->updateStatus(); // for previous package
+
+		QString pkgname = line.mid(9);
+		if( pkgname==0 || pkgname == "" ) {
+			iError = tr("Empty package name");
+			//std::cout << "Encountered empty package name!" << std::endl;
+			return false;
+		}
+		if( dpkg )
+			pkg = iPackages.value(pkgname,0);
+		if( !dpkg || pkg==0 ) {
+			pkg = new Package(pkgname, this);
+			iPackages.insert(pkgname,pkg);
+			if( dpkg ) {
+				pkg->setLocal(true);
+				//std::cout << pkgname.toStdString() << std::endl;
+			}
+		}
+		if( dpkg )
+			pkg->setInstalled(true);
+		multiline=MultiLineNone;
+	}
+
+	if( line.startsWith("Status: ") && pkg != 0 && dpkg )
+	{
+		if( line.mid(8) == "install ok installed" )
+			pkg->setInstalled(true);
+		else
+			pkg->setInstalled(false);
+	}
+	if( line.startsWith("Section: ") && pkg != 0 && pkg->section()=="" )
+	{
+		pkg->setSection( line.mid(9) );
+	}
+	if( line.startsWith("Version: ") && pkg != 0 )
+	{
+		if( dpkg )
+			pkg->setVersionInstalled( line.mid(9) );
+		else
+			pkg->setVersionAvailable( line.mid(9) );
+	}
+	if( line.startsWith("Size: ") && pkg != 0 && pkg->size()==0 )
+	{
+		pkg->setSize( line.mid(6).toInt() );
+	}
+	if( line.startsWith("Installed-Size: ") && pkg != 0 && pkg->installedSize()==0 )
+	{
+		pkg->setInstalledSize( line.mid(16).toInt() );
+	}
+	if( line.startsWith("Maemo-Display-Name: ") && pkg != 0 && pkg->maemoDisplayName()=="" )
+	{
+		pkg->setMaemoDisplayName( line.mid(20) );
+	}
+
+	if( multiline == MultiLineDesc ) {
+		if( line.startsWith(" ") && line.trimmed()!="" ) {
+			pkg->appendDescLong( line.trimmed() + " " );
+		} else {
+			multiline = MultiLineNone;
+		}
+	}
+	if( multiline == MultiLineIcon ) {
+		if( line.startsWith(" ") && line.trimmed()!="" ) {
+			pkg->appendIconData( line.trimmed() );
+		} else {
+			multiline = MultiLineNone;
+		}
+	}
+
+	if( line.startsWith("Description: ") && pkg != 0 && pkg->descShort()=="" )
+	{
+		pkg->setDescShort( line.mid(13) );
+		multiline = MultiLineDesc;
+	}
+	if( line.startsWith("Maemo-Icon-26:") && pkg != 0 && !pkg->hasIconData() )
+	{
+		if( line.mid(15).trimmed() != "" ) {
+			pkg->appendIconData( line.mid(15).trimmed() );
+		}
+		multiline = MultiLineIcon;
+	}
 
 	return true;
 }
@@ -345,63 +380,69 @@ bool AptInterface::parseSimulated()
 	return true;
 }
 
-void AptInterface::simulateInstall(QStringList names_)
+void AptInterface::simulateInstall()
 {
-	pkgNamesList = names_;
+	//pkgNamesList = names_;
 	iMainWindow->busyDialog(true, "Operation in progress", "Reading dependencies");
-	/*
-	std::cout << "SIMULATE apt-get install: ";
-	for( int i=0; i<names_.count(); i++)
-	{
-		std::cout << names_.at(i).toStdString() + " ";
-	}
-	std::cout << std::endl;
-	*/
 
 	iMode = ModeAptGetSimulateInstall;
 
+	pkgNamesList.clear();
+	QHashIterator<QString, Package*> i( iPackages );
+	while (i.hasNext())
+	{
+		i.next();
+		if( i.value()->isMarkedForOperation() )
+		{
+			QString pkgname = i.value()->name();
+			if( i.value()->markedOperation() == Package::PkgOpRemove )
+				pkgname.append("-");
+			pkgNamesList << pkgname;
+		}
+	}
+
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
 	runParameters << "-qsy" << "install";
-	runParameters << names_;
+	runParameters << pkgNamesList;
 
 	iRunProc->start(runBinary,runParameters);
 }
 
-void AptInterface::simulateRemove(QStringList names_)
+/*
+void AptInterface::simulateRemove()
 {
-	pkgNamesList = names_;
 	iMainWindow->busyDialog(true, "Operation in progress", "Reading dependencies");
-	/*
-	std::cout << "SIMULATE apt-get remove: ";
-	for( int i=0; i<names_.count(); i++)
-	{
-		std::cout << names_.at(i).toStdString() + " ";
-	}
-	std::cout << std::endl;
-	*/
 
 	iMode = ModeAptGetSimulateRemove;
 
+	pkgNamesList.clear();
+	QHashIterator<QString, Package*> i( iPackages );
+	while (i.hasNext())
+	{
+		i.next();
+		if( i.value()->markedOperation() == Package::PkgOpRemove )
+			pkgNamesList << i.value()->name();
+	}
+
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
 	runParameters << "-qsy" << "remove";
-	runParameters << names_;
+	runParameters << pkgNamesList;
 
 	iRunProc->start(runBinary,runParameters);
 }
+*/
 
-void AptInterface::installPackages(QStringList names_)
+void AptInterface::installPackages(QStringList names_, int inst_, int remv_)
 {
-	iMainWindow->busyDialog(true, "Operation in progress", "Downloading and installing packages");
-	/*
-	std::cout << "apt-get install ";
-	for( int i=0; i<names_.count(); i++)
-	{
-		std::cout << names_.at(i).toStdString() + " ";
-	}
-	std::cout << std::endl;
-	*/
+	QString busytext = "";
+	if( inst_ > 0 )
+		busytext += QString("Downloading and installing %0 package(s)<br>").arg(inst_);
+	if( remv_ > 0 )
+		busytext += QString("Removing %0 package(s)<br>").arg(remv_);
+
+	iMainWindow->busyDialog(true, "Operation in progress", busytext);
 
 	iMode = ModeAptGetInstall;
 
@@ -413,17 +454,10 @@ void AptInterface::installPackages(QStringList names_)
 	iRunProc->start(runBinary,runParameters);
 }
 
+/*
 void AptInterface::removePackages(QStringList names_)
 {
 	iMainWindow->busyDialog(true, "Operation in progress", "Removing packages");
-	/*
-	std::cout << "apt-get remove ";
-	for( int i=0; i<names_.count(); i++)
-	{
-		std::cout << names_.at(i).toStdString() + " ";
-	}
-	std::cout << std::endl;
-	*/
 
 	iMode = ModeAptGetRemove;
 
@@ -434,6 +468,7 @@ void AptInterface::removePackages(QStringList names_)
 
 	iRunProc->start(runBinary,runParameters);
 }
+*/
 
 void AptInterface::aptGetClean()
 {
@@ -461,6 +496,27 @@ void AptInterface::logToFile( QByteArray data )
 	}
 }
 
+QString AptInterface::readLogFile()
+{
+	QString log = "";
+
+	QFile own("/root/.fapman/lastlog.txt");
+	if( own.open(QIODevice::ReadOnly | QIODevice::Text ) )
+	{
+		while(!own.atEnd())
+		{
+			QString line = own.readLine();
+			log.append( line );
+		}
+		own.close();
+	}
+
+	if( log=="" )
+		log = "The log is empty";
+
+	return log;
+}
+
 QStringList AptInterface::getRepositories()
 {
 	QStringList repos;
