@@ -36,17 +36,22 @@ class AptInterface : public QObject
 	Q_OBJECT
 
 public:
+	enum aptMode { ModeNone, ModeReadInfo, ModeAptGetUpdate, ModeAptGetInstall, ModeAptGetRemove,
+				   ModeAptGetSimulateInstall, ModeAptGetSimulateRemove, ModeAptGetClean };
+
 	AptInterface(MainWindow* w_);
 	~AptInterface();
 
+	enum multiLine { MultiLineNone, MultiLineDesc, MultiLineIcon };
+
 	void setPackageView(PackageView* p_) { iWinPackageView = p_; }
 	bool isInterfaceReady();
 	bool updateCatalogs();
 	bool updatePackageList();
-	void installPackages(QStringList names_);
-	void removePackages(QStringList names_);
-	void simulateInstall(QStringList names_);
-	void simulateRemove(QStringList names_);
+	void installPackages(QStringList names_, int inst_, int remv_);
+	//void removePackages(QStringList names_);
+	void simulateInstall();
+	//void simulateRemove();
 	void aptGetClean();
 	int selectedCount() { return iSelectedCount; }
 	void increaseSelected() { iSelectedCount++; }
@@ -57,17 +62,17 @@ public:
 	void getOriginalPackageList(QStringList& names_) { names_=pkgNamesList; }
 
 	void logToFile( QByteArray data );
+	QString readLogFile();
 
 	QStringList getRepositories(void);
 	bool writeRepositories(QStringList repos);
 	bool needRepoRefresh();
 
-	enum aptMode { ModeNone, ModeReadInfo, ModeAptGetUpdate, ModeAptGetInstall, ModeAptGetRemove,
-				   ModeAptGetSimulateInstall, ModeAptGetSimulateRemove, ModeAptGetClean };
 
 private:
 	bool parseAptCacheOutput();
 	bool readDpkgDatabase();
+	bool processPackageDataLine(Package*& pkg, QByteArray& line, multiLine& multiline, bool dpkg);
 	bool parseSimulated();
 
 	QHash<QString, Package*> iPackages;
