@@ -31,10 +31,15 @@ dimmer::dimmer(QWidget *parent) :
 	iLabel->setAlignment(Qt::AlignCenter);
 	iLayout->addWidget(iLabel);
 	iBusy = false;
-	iTimer = new QTimer(this);
-	connect(iTimer,SIGNAL(timeout()),this,SLOT(repaint()));
 	iAnim = 0;
 	iAnimDir = 1;
+	iAnimY = this->rect().height()-this->rect().height()/4;
+
+	iTimer = new QTimer(this);
+	connect(iTimer,SIGNAL(timeout()),this,SLOT(timerEvent()));
+
+	connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged()));
+
 	hide();
 }
 
@@ -53,7 +58,7 @@ void dimmer::paintEvent(QPaintEvent *)
 	int animMax = 10;
 
 	painter.drawEllipse(QPoint(this->rect().left()+(this->rect().width()/2-(animMax*step)/2)+iAnim*step,
-							   this->rect().height()-this->rect().height()/4), 10, 10);
+							   iAnimY), 10, 10);
 
 	iAnim += iAnimDir;
 	if( iAnim>animMax )
@@ -63,11 +68,22 @@ void dimmer::paintEvent(QPaintEvent *)
 
 }
 
+void dimmer::timerEvent()
+{
+	repaint(0,iAnimY-10,rect().width(),20);
+}
+
 void dimmer::resizeEvent(QResizeEvent *)
 {
 	this->resize(iParent->size());
 	iLayout->setGeometry(iParent->rect());
 	iLayout->setSizeConstraint(QLayout::SetMaximumSize);
+	iAnimY = this->rect().height()-this->rect().height()/4;
+}
+
+void dimmer::orientationChanged()
+{
+	resizeEvent(0);
 }
 
 void dimmer::dim(QString title, QString message)
@@ -86,4 +102,3 @@ void dimmer::undim()
 	iTimer->stop();
 	hide();
 }
-
