@@ -19,6 +19,7 @@
 
 #include <QtGui/QApplication>
 #include "mainwindow.h"
+#include "confirmdialog.h"
 
 int main(int argc, char *argv[])
 {
@@ -30,5 +31,14 @@ int main(int argc, char *argv[])
     w.show();
 #endif
 
+#ifdef Q_WS_MAEMO_5
+	uid_t userUID = getuid();
+	if( userUID != 0 ) {
+		ConfirmDialog d(false, &w);
+		d.setText("Warning", "You are not running the application as root. It might not work as intended.");
+		d.exec();
+	}
+#endif
+
     return a.exec();
 }
