@@ -23,6 +23,7 @@
 
 #ifdef Q_WS_MAEMO_5
 #include <QtMaemo5>
+#include <unistd.h>
 #endif
 
 #include "mainwindow.h"
@@ -33,6 +34,7 @@
 #include "confirmdialog.h"
 #include "dimmer.h"
 #include "repoview.h"
+#include "help.h"
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
@@ -56,6 +58,7 @@ MainWindow::MainWindow(QWidget *parent) :
 
 	iReposAutoUpdating = false;
 	iUpgradeAutoUpdate = true;
+
 }
 
 MainWindow::~MainWindow()
@@ -100,6 +103,9 @@ void MainWindow::on_btnListInstalled_clicked()
 	if( iOperation != OpNone ) return;
 	iOperation = OpUninstall;
 
+	iWinPackageView->resetWindow();
+	iWinPackageView->setStatFilter( Package::PkgStatInstalled );
+
 	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
 	if( !iAptInterface->updatePackageList() )
 	{
@@ -114,6 +120,9 @@ void MainWindow::on_btnListInstallable_clicked()
 	if( iOperation != OpNone ) return;
 	iOperation = OpInstall;
 
+	iWinPackageView->resetWindow();
+	iWinPackageView->setStatFilter( Package::PkgStatNotInstalled );
+
 	if( iAptInterface->needRepoRefresh() )
 	{
 		iReposAutoUpdating = true;
@@ -135,6 +144,9 @@ void MainWindow::on_btnUpgrade_clicked()
 	if( iOperation != OpNone ) return;
 	iOperation = OpUpgrade;
 
+	iWinPackageView->resetWindow();
+	iWinPackageView->setStatFilter( Package::PkgStatUpgradeable );
+
 	if( iAptInterface->needRepoRefresh() ) //|| iUpgradeAutoUpdate ) // this feature is disabled on purpose
 	{
 		iReposAutoUpdating = true;
@@ -174,15 +186,25 @@ void MainWindow::operationFinished(AptInterface::aptMode lastMode_, bool success
 					  "%0 to install/upgrade, %1 to remove<br>").arg(inst.count()).arg(remv.count());
 
 			if( inst.count()>0 ) {
-				pkglist += "<br><b><u>INSTALL/UPDATE:</u></b><br>";
-				for( int i=0; i<inst.count(); i++ )
-					pkglist += inst.at(i) + "<br>";
+				pkglist += "<br><b><u>INSTALL/UPGRADE:</u></b><br>";
+				for( int i=0; i<inst.count(); i++ ) {
+					pkglist += inst.at(i);
+					Package* pkg = iAptInterface->packages()->value(inst.at(i),0);
+					if( pkg!=0 && pkg->size()>0 )
+						pkglist += QString(" (%0 kB)").arg(pkg->size()/1024);
+					pkglist += "<br>";
+				}
 			}
 
 			if( remv.count()>0 ) {
 				pkglist += "<br><b><u>REMOVE:</u></b><br>";
-				for( int i=0; i<remv.count(); i++ )
-					pkglist += remv.at(i) + "<br>";
+				for( int i=0; i<remv.count(); i++ ) {
+					pkglist += remv.at(i);
+					Package* pkg = iAptInterface->packages()->value(remv.at(i),0);
+					if( pkg!=0 && pkg->installedSize()>0 )
+						pkglist += QString(" (%0 kB)").arg(pkg->installedSize()/1024);
+					pkglist += "<br>";
+				}
 			}
 
 			busyDialog(false);
@@ -193,10 +215,10 @@ void MainWindow::operationFinished(AptInterface::aptMode lastMode_, bool success
 				//std::cout << "*** Proceeding with install/remove operation" << std::endl;
 				QStringList names;
 				iAptInterface->getOriginalPackageList(names);
-				if( lastMode_ == AptInterface::ModeAptGetSimulateInstall )
-					iAptInterface->installPackages(names);
-				if( lastMode_ == AptInterface::ModeAptGetSimulateRemove )
-					iAptInterface->removePackages(names);
+				if( lastMode_ == AptInterface::ModeAptGetSimulateInstall || lastMode_ == AptInterface::ModeAptGetSimulateRemove )
+					iAptInterface->installPackages(names, inst.count(), remv.count());
+				//if( lastMode_ == AptInterface::ModeAptGetSimulateRemove )
+				//	iAptInterface->removePackages(names);
 			} else {
 				//std::cout << "*** Installation/removal cancelled" << std::endl;
 			}
@@ -211,10 +233,7 @@ void MainWindow::operationFinished(AptInterface::aptMode lastMode_, bool success
 			busyDialog(false);
 			iWinPackageView->hide();
 			ConfirmDialog d(false, this);
-			if( lastMode_ == AptInterface::ModeAptGetInstall )
-				d.setText("Operation finished","Package(s) successfully installed");
-			else if( lastMode_ == AptInterface::ModeAptGetRemove )
-				d.setText("Operation finished","Package(s) successfully removed");
+			d.setText("Operation finished","Package operations were performed successfully");
 			d.exec();
 			return;
 		}
@@ -268,6 +287,7 @@ void MainWindow::busyDialog(bool visible_, QString title, QString text)
 {
 	if( visible_ ) {
 		ui->menuMenu->setEnabled(false);
+		ui->centralWidget->setEnabled(false);
 		iWinPackageView->disableMenu();
 		iDimmer->resizeEvent(0);
 		iDimmer->dim(title, text);
@@ -277,6 +297,7 @@ void MainWindow::busyDialog(bool visible_, QString title, QString text)
 		iDimmer->undim();
 		iWinPackageView->dim()->undim();
 		ui->menuMenu->setEnabled(true);
+		ui->centralWidget->setEnabled(true);
 		iWinPackageView->enableMenu();
 	}
 }
@@ -305,3 +326,12 @@ void MainWindow::closeEvent(QCloseEvent *event)
 		event->accept();
 	}
 }
+
+void MainWindow::on_actionView_log_triggered()
+{
+	ConfirmDialog d(false, this);
+	QString log = iAptInterface->readLogFile();
+	log.replace('\n',"<br>");
+	d.setText("Log", "<font size=\"-1\">" + log + "</font>" );
+	d.exec();
+}
